#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import pandas as pd

# First solution is to use pandas. It's fast and easy but some attention is needed.
# For the read csv, separator needs to be specified
data_sales = pd.read_csv("sales_by_customer_month_store.csv", sep=';')
data_customer = pd.read_csv("customer_info.csv", sep=';')
# Cutting users not from USA
data_customer = data_customer.loc[data_customer["country"] == "USA"]
# Joining table has to be done with the merge function
data_join = pd.merge(data_customer,data_sales, on="customer_id")
# We want to know how much females and males spent during a year on average each month
# So we either group two times, first we sum and then we do the average
data_result_1 = data_join.groupby(["store_id","gender","month_of_year"]).agg({"total_sales":"sum"})
data_result_1 = data_result_1.groupby(["store_id","gender"]).agg({"total_sales":"mean"})
data_result_1 = data_result_1.rename(index=str, columns={"total_sales": "yearly_average"})
print(data_result_1.to_string())
print("\n")
# Or we can group only once and then simply divide by the count of distinct months
data_result_2 = data_join.groupby(["store_id","gender"]).agg({"total_sales":"sum", "month_of_year":"nunique"})
data_result_2["yearly_average"] = data_result_2["total_sales"]/data_result_2["month_of_year"]
print(data_result_2[["yearly_average"]].to_string())
print("\n")

# Second solution: manage files on our own
import csv
from pprint import pprint
#open files and use dictreader to cache information
file1 = open("sales_by_customer_month_store.csv", mode='r', encoding='utf-8-sig')
csv_sales = csv.DictReader(file1, delimiter = ";")
file2 = open("customer_info.csv", mode='r', encoding='utf-8-sig')
csv_customer = csv.DictReader(file2, delimiter = ";")
#select only customers born in USA
customers_usa_gender = dict()
for line in csv_customer:
    if line["country"] == "USA":
        c_id = int(line["customer_id"])
        customers_usa_gender[c_id] = line["gender"]
file2.close()
#groupby and aggregation
store_gender_info = dict()
for line in csv_sales:
    c_id = int(line["customer_id"])
    if c_id in customers_usa_gender:
        s_id = int(line["store_id"])
        if s_id not in store_gender_info:
            store_gender_info[s_id] = dict()
        gender_info = store_gender_info[s_id]
        gender = customers_usa_gender[c_id]
        if gender not in gender_info:
            gender_info[gender] = {"total_sales" : 0, "distinct_months" : set()}
        m_y = int(line["month_of_year"])
        gender_info[gender]["total_sales"] += float(line["total_sales"])
        gender_info[gender]["distinct_months"].add(m_y)
file1.close()
#caclulating yearly average
for store in store_gender_info:
    for gender in store_gender_info[store]:
        info = store_gender_info[store][gender]
        info["yearly_average"] = info["total_sales"]/len(info["distinct_months"])
        keys = ["yearly_average"]
        new_dict = { key: info[key] for key in keys }
        store_gender_info[store][gender] = new_dict
#printing result
pprint(store_gender_info)
